@@ -610,6 +610,11 @@ class RedHat5Hostname(Hostname):
     distribution = 'Redhat'
     strategy_class = RedHatStrategy
 
+class RHELHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Red hat enterprise linux'
+    strategy_class = RedHatStrategy
+
 class RedHatServerHostname(Hostname):
     platform = 'Linux'
     distribution = 'Red hat enterprise linux server'
