@@ -19,10 +19,9 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from ansible.compat.six import string_types
-
+from ansible import constants as C
 from ansible.errors import AnsibleParserError
-
+from ansible.module_utils.six import string_types
 from ansible.playbook.attribute import FieldAttribute
 from ansible.playbook.base import Base
 from ansible.playbook.become import Become
@@ -30,7 +29,7 @@ from ansible.playbook.block import Block
 from ansible.playbook.helpers import load_list_of_blocks, load_list_of_roles
 from ansible.playbook.role import Role
 from ansible.playbook.taggable import Taggable
-from ansible.vars import preprocess_vars
+from ansible.vars.manager import preprocess_vars
 
 try:
     from __main__ import display
@@ -55,40 +54,40 @@ class Play(Base, Taggable, Become):
     """
 
     # =================================================================================
-    # Connection-Related Attributes
+    _name = FieldAttribute(isa='string', default='', always_post_validate=True)
 
     # TODO: generalize connection
-    _accelerate          = FieldAttribute(isa='bool', default=False, always_post_validate=True)
-    _accelerate_ipv6     = FieldAttribute(isa='bool', default=False, always_post_validate=True)
-    _accelerate_port     = FieldAttribute(isa='int', default=5099, always_post_validate=True)
+    _accelerate = FieldAttribute(isa='bool', default=False, always_post_validate=True)
+    _accelerate_ipv6 = FieldAttribute(isa='bool', default=False, always_post_validate=True)
+    _accelerate_port = FieldAttribute(isa='int', default=5099, always_post_validate=True)
 
     # Connection
-    _gather_facts        = FieldAttribute(isa='bool', default=None, always_post_validate=True)
-    _gather_subset       = FieldAttribute(isa='barelist', default=None, always_post_validate=True)
-    _gather_timeout      = FieldAttribute(isa='int', default=None, always_post_validate=True)
-    _hosts               = FieldAttribute(isa='list', required=True, listof=string_types, always_post_validate=True)
-    _name                = FieldAttribute(isa='string', default='', always_post_validate=True)
+    _fact_path = FieldAttribute(isa='string', default=None)
+    _gather_facts = FieldAttribute(isa='bool', default=None, always_post_validate=True)
+    _gather_subset = FieldAttribute(isa='barelist', default=None, always_post_validate=True)
+    _gather_timeout = FieldAttribute(isa='int', default=None, always_post_validate=True)
+    _hosts = FieldAttribute(isa='list', required=True, listof=string_types, always_post_validate=True)
 
     # Variable Attributes
-    _vars_files          = FieldAttribute(isa='list', default=[], priority=99)
-    _vars_prompt         = FieldAttribute(isa='list', default=[], always_post_validate=True)
-    _vault_password      = FieldAttribute(isa='string', always_post_validate=True)
+    _vars_files = FieldAttribute(isa='list', default=[], priority=99)
+    _vars_prompt = FieldAttribute(isa='list', default=[], always_post_validate=True)
+    _vault_password = FieldAttribute(isa='string', always_post_validate=True)
 
     # Role Attributes
-    _roles               = FieldAttribute(isa='list', default=[], priority=90)
+    _roles = FieldAttribute(isa='list', default=[], priority=90)
 
     # Block (Task) Lists Attributes
-    _handlers            = FieldAttribute(isa='list', default=[])
-    _pre_tasks           = FieldAttribute(isa='list', default=[])
-    _post_tasks          = FieldAttribute(isa='list', default=[])
-    _tasks               = FieldAttribute(isa='list', default=[])
+    _handlers = FieldAttribute(isa='list', default=[])
+    _pre_tasks = FieldAttribute(isa='list', default=[])
+    _post_tasks = FieldAttribute(isa='list', default=[])
+    _tasks = FieldAttribute(isa='list', default=[])
 
     # Flag/Setting Attributes
-    _any_errors_fatal    = FieldAttribute(isa='bool', default=False, always_post_validate=True)
-    _force_handlers      = FieldAttribute(isa='bool', always_post_validate=True)
+    _force_handlers = FieldAttribute(isa='bool', always_post_validate=True)
     _max_fail_percentage = FieldAttribute(isa='percent', always_post_validate=True)
-    _serial              = FieldAttribute(isa='list', default=[], always_post_validate=True)
-    _strategy            = FieldAttribute(isa='string', default='linear', always_post_validate=True)
+    _serial = FieldAttribute(isa='list', default=[], always_post_validate=True)
+    _strategy = FieldAttribute(isa='string', default=C.DEFAULT_STRATEGY, always_post_validate=True)
+    _order = FieldAttribute(isa='string', always_post_validate=True)
 
     # =================================================================================
 
@@ -122,7 +121,7 @@ class Play(Base, Taggable, Become):
         Adjusts play datastructure to cleanup old/legacy items
         '''
 
-        assert isinstance(ds, dict)
+        assert isinstance(ds, dict), 'while preprocessing data (%s), ds should be a dict but was a %s' % (ds, type(ds))
 
         # The use of 'user' in the Play datastructure was deprecated to
         # line up with the same change for Tasks, due to the fact that
@@ -131,8 +130,8 @@ class Play(Base, Taggable, Become):
             # this should never happen, but error out with a helpful message
             # to the user if it does...
             if 'remote_user' in ds:
-                raise AnsibleParserError("both 'user' and 'remote_user' are set for %s."
-                        " The use of 'user' is deprecated, and should be removed" % self.get_name(), obj=ds)
+                raise AnsibleParserError("both 'user' and 'remote_user' are set for %s. "
+                                         "The use of 'user' is deprecated, and should be removed" % self.get_name(), obj=ds)
 
             ds['remote_user'] = ds['user']
             del ds['user']
@@ -146,8 +145,8 @@ class Play(Base, Taggable, Become):
         '''
         try:
             return load_list_of_blocks(ds=ds, play=self, variable_manager=self._variable_manager, loader=self._loader)
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading tasks", obj=self._ds, orig_exc=e)
 
     def _load_pre_tasks(self, attr, ds):
         '''
@@ -156,8 +155,8 @@ class Play(Base, Taggable, Become):
         '''
         try:
             return load_list_of_blocks(ds=ds, play=self, variable_manager=self._variable_manager, loader=self._loader)
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading pre_tasks", obj=self._ds, orig_exc=e)
 
     def _load_post_tasks(self, attr, ds):
         '''
@@ -166,8 +165,8 @@ class Play(Base, Taggable, Become):
         '''
         try:
             return load_list_of_blocks(ds=ds, play=self, variable_manager=self._variable_manager, loader=self._loader)
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading post_tasks", obj=self._ds, orig_exc=e)
 
     def _load_handlers(self, attr, ds):
         '''
@@ -176,8 +175,8 @@ class Play(Base, Taggable, Become):
         '''
         try:
             return load_list_of_blocks(ds=ds, play=self, use_handlers=True, variable_manager=self._variable_manager, loader=self._loader)
-        except AssertionError:
-            raise AnsibleParserError("A malformed block was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed block was encountered while loading handlers", obj=self._ds, orig_exc=e)
 
     def _load_roles(self, attr, ds):
         '''
@@ -190,8 +189,8 @@ class Play(Base, Taggable, Become):
 
         try:
             role_includes = load_list_of_roles(ds, play=self, variable_manager=self._variable_manager, loader=self._loader)
-        except AssertionError:
-            raise AnsibleParserError("A malformed role declaration was encountered.", obj=self._ds)
+        except AssertionError as e:
+            raise AnsibleParserError("A malformed role declaration was encountered.", obj=self._ds, orig_exc=e)
 
         roles = []
         for ri in role_includes:
@@ -203,17 +202,17 @@ class Play(Base, Taggable, Become):
         vars_prompts = []
         for prompt_data in new_ds:
             if 'name' not in prompt_data:
-                display.deprecated("Using the 'short form' for vars_prompt has been deprecated")
+                display.deprecated("Using the 'short form' for vars_prompt has been deprecated", version="2.7")
                 for vname, prompt in prompt_data.items():
                     vars_prompts.append(dict(
-                        name      = vname,
-                        prompt    = prompt,
-                        default   = None,
-                        private   = None,
-                        confirm   = None,
-                        encrypt   = None,
-                        salt_size = None,
-                        salt      = None,
+                        name=vname,
+                        prompt=prompt,
+                        default=None,
+                        private=None,
+                        confirm=None,
+                        encrypt=None,
+                        salt_size=None,
+                        salt=None,
                     ))
             else:
                 vars_prompts.append(prompt_data)
